﻿using System;
using System.Diagnostics;
using GHIElectronics.TinyCLR.Devices.Adc;
using GHIElectronics.TinyCLR.Devices.Gpio;
using GHIElectronics.TinyCLR.Devices.Pwm;
using GHIElectronics.TinyCLR.Pins;
using System.Threading;
using Webge.TCLR.Displays;

namespace Incubateur
{
    class Program
    {
        enum Etat : ushort // Incubateur V2
        {
            chaufferOn = 0,
            chaufferOff = 1
        }

        static bool Alarmelum = false; //  plus loin

        static void Main()
        {
            // Variables
            var Ntemperature = 0; // Incubateur V1
            var Temperature = 0.0; // Incubateur V1
            var Tconsigne = 26.0; // Incubateur V2
            var ecart = 0.1; // Incubateur V2
            var etat = Etat.chaufferOff; // Incubateur V2
            var ecartAlarme = 2;  // Incubateur V3

            // Création des objets pour une carte FEZ T18
            // Pour une carte FEZ PANDA 3, remplacer FEZ par FEZPandaIII
            //-----------------------------------------------------------------
            // ADC -> création de l'objet ModuleMesureTemp
            var adc = AdcController.GetDefault();
            var ModuleMesureTemp = adc.OpenChannel(FEZ.AdcChannel.A0);

            // PWM -> création de l'objet Alarme
            var pwm = PwmController.FromName(FEZ.PwmChannel.Controller3.Id);
            var Alarme = pwm.OpenChannel(FEZ.PwmChannel.Controller3.D11);
            pwm.SetDesiredFrequency(2000);
            Alarme.SetActiveDutyCyclePercentage(0.5);

            // GPIO -> création de l'objet CdeChauffage
            var gpio = GpioController.GetDefault();
            var CdeChauffage = gpio.OpenPin(FEZ.GpioPin.D6);
            CdeChauffage.SetDriveMode(GpioPinDriveMode.Output);
            CdeChauffage.Write(GpioPinValue.Low);

            // Afficheur
            var lcd = new ELCD162();

            // Thread
            var threadLed = new Thread(FlashLed);  // plus loin

            // Initialisation
            //-----------------------------------------------------------------
            lcd.Init(); lcd.CursorOff();
            threadLed.Start();  // plus loin

 
            while (true)
            {
                // Lire la température
                //-----------------------------------------------------------------
                Ntemperature = ModuleMesureTemp.ReadValue(); // Incubateur V1

                // Traiter
                //-----------------------------------------------------------------
                Temperature = 0.0185 * Ntemperature - 20.5; // Incubateur V1

                // Alarme
                if ((Temperature > (Tconsigne + ecartAlarme)) || (Temperature < (Tconsigne - ecartAlarme)))
                {
                    Alarme.Start();
                    Alarmelum = true; //  plus loin
                }
                else
                {
                    Alarme.Stop();
                    Alarmelum = false; //  plus loin
                }

                // Chauffage (Incubateur V2)
                switch (etat)
                {
                    case Etat.chaufferOn:
                        CdeChauffage.Write(GpioPinValue.High);
                        if (Temperature >= (Tconsigne + ecart))
                        {
                            etat = Etat.chaufferOff;
                        }
                        break;
                    case Etat.chaufferOff:
                        CdeChauffage.Write(GpioPinValue.Low);
                        if (Temperature <= (Tconsigne - ecart))
                        {
                            etat = Etat.chaufferOn;
                        }
                        break;
                    default:
                        break;
                }

                // Afficher la température sur le lcd
                //-----------------------------------------------------------------
                lcd.ClearScreen();
                lcd.PutString("T = " + Temperature.ToString("F1") + "C"); // Incubateur V1
                lcd.SetCursor(0, 1);
                lcd.PutString("Tcons = " + Tconsigne.ToString("F1") + "C"); // Incubateur V1
                Debug.WriteLine("Ntemperature" + Ntemperature.ToString("F0"));
                Debug.WriteLine("Temperature" + Temperature.ToString("F1") + "°C");
                Thread.Sleep(1000);
            }

        }

        #region Thread Ledflash
        static void FlashLed() // plus loin
        {
            var pwm2 = PwmController.FromName(FEZ.PwmChannel.Controller3.Id);
            var ledAlarme = pwm2.OpenChannel(FEZ.PwmChannel.Controller3.D9);
            pwm2.SetDesiredFrequency(1000);

            while (true)
            {
                if (Alarmelum)
                {
                    ledAlarme.Start();
                    for (var i = 0.0; i < 100.0; i++)
                    {
                        ledAlarme.SetActiveDutyCyclePercentage(i / 100);
                        Thread.Sleep(20);
                    }
                }
                else
                {
                    ledAlarme.Stop();
                }

            }
        }
        #endregion
    }
}
